/// <reference types="node" />
import { ErrorChain } from "container.ts/lib/error";
import * as debug from "debug";
import { Observable, Subject, TimeoutError } from "rxjs";
import * as pdu from "../pdu";
/** Master error codes. */
export declare enum EMasterError {
    Timeout = 0,
    Write = 1
}
/** Modbus ADU error class. */
export declare class MasterError extends ErrorChain {
    protected readonly isMasterError = true;
    constructor(code: EMasterError, cause?: Error, context?: object);
}
/** Master log names. */
export declare enum EMasterLog {
    Request = "ModbusMasterRequest",
    Response = "ModbusMasterResponse",
    Exception = "ModbusMasterException",
    Error = "MasterMasterError"
}
/** Master log interface. */
export declare class MasterLog<Req extends pdu.Request, Res extends pdu.Response, Exc extends pdu.Exception> {
    protected readonly debug: debug.Debugger;
    request(request: Req, errorCount: number): void;
    response(response: Res): void;
    exception(exception: Exc): void;
    error(error: any): void;
    bytesTransmitted(value: number): void;
    bytesReceived(value: number): void;
}
/** Conditional retry callback type. */
export declare type IMasterRetryWhen<Req extends pdu.Request, Res extends pdu.Response, Exc extends pdu.Exception> = (master: Master<Req, Res, Exc>, retry: number, errorCount: number, error: any, request?: Req) => void;
/** Modbus ADU master request options. */
export interface IMasterRequestOptions<Req extends pdu.Request, Res extends pdu.Response, Exc extends pdu.Exception, L extends MasterLog<Req, Res, Exc> = MasterLog<Req, Res, Exc>> {
    retry?: number;
    timeout?: number;
    retryWhen?: IMasterRetryWhen<Req, Res, Exc>;
    log?: L;
}
/** Internal queue in type. */
export interface IMasterQueueIn<Req extends pdu.Request, Res extends pdu.Response, Exc extends pdu.Exception, L extends MasterLog<Req, Res, Exc> = MasterLog<Req, Res, Exc>> {
    id: number;
    request: Req;
    options: IMasterRequestOptions<Req, Res, Exc, L>;
}
/** Internal queue out type. */
export interface IMasterQueueOut<Res extends pdu.Response> {
    id: number;
    response?: Res;
    error?: any;
}
/** Modbus abstract ADU master. */
export declare abstract class Master<Req extends pdu.Request, Res extends pdu.Response, Exc extends pdu.Exception, L extends MasterLog<Req, Res, Exc> = MasterLog<Req, Res, Exc>> {
    /** Returns true if error is RxJS Timeout. */
    static isTimeoutError(error: any): error is TimeoutError;
    /** Returns true if error is MasterError. */
    static isMasterError(error: any): error is MasterError;
    /** Default number of retries performed during requests. */
    readonly retry: number;
    /** Default timeout of requests. */
    readonly timeout: number;
    /** Default retryWhen callback of requests. */
    readonly retryWhen: IMasterRetryWhen<Req, Res, Exc>;
    /** Responses/exceptions stream. */
    receive: Subject<Res | Exc>;
    /** Requests stream. */
    transmit: Subject<Req>;
    /** Internal log interface. */
    protected readonly log: L;
    /** Internal transmit input queue. */
    protected queueIn: Subject<IMasterQueueIn<Req, Res, Exc, L>>;
    /** Internal transmit output queue. */
    protected queueOut: Subject<IMasterQueueOut<Res>>;
    /** Queue ID counter. */
    protected queueCounter: number;
    /** Internal buffer. */
    protected buffer: Buffer;
    constructor(options: IMasterRequestOptions<Req, Res, Exc, L>, logConstructor?: any);
    /**
     * This function code is used to read from 1 to 2000 contiguous status of coils in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfCoils Quantity of coils.
     * @param options Request options.
     */
    readCoils(startingAddress: number, quantityOfCoils: number, options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * This function code is used to read from 1 to 2000 contiguous status of discrete inputs in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfInputs Quantity of inputs.
     * @param options Request options.
     */
    readDiscreteInputs(startingAddress: number, quantityOfInputs: number, options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * This function code is used to read  the contents of a contiguous block of holding registers in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfRegisters Quantity of registers.
     * @param options Request options.
     */
    readHoldingRegisters(startingAddress: number, quantityOfRegisters: number, options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * This function code is used to read from 1 to 125 contiguous input registers in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfRegisters Quantity of registers.
     * @param options Request options.
     */
    readInputRegisters(startingAddress: number, quantityOfRegisters: number, options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * This function code is used to write a single output to either ON or OFF in a remote device.
     * @param outputAddress Output address.
     * @param outputValue  Output value.
     * @param options Request options.
     */
    writeSingleCoil(outputAddress: number, outputValue: boolean, options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * This function code is used to write a single holding register in a remote device.
     * @param registerAddress Register address.
     * @param registerValue Register value.
     * @param options Request options.
     */
    writeSingleRegister(registerAddress: number, registerValue: number, options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * This function code is used to force each coil in a sequence of coils to either ON or OFF in a remote device.
     * @param startingAddress Starting address.
     * @param outputValues Output values.
     * @param options Request options.
     */
    writeMultipleCoils(startingAddress: number, outputValues: boolean[], options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * This function code is used to write a block of contiguous registers (1 to 123 registers) in a remote device.
     * @param startingAddress Starting address.
     * @param registerValues Register values.
     * @param options Request options.
     */
    writeMultipleRegisters(startingAddress: number, registerValues: number[], options?: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /**
     * Default retryWhen callback for conditional retries.
     * This will allow retries in cases of timeout and throw all other errors.
     */
    masterDefaultRetryWhen(master: Master<Req, Res, Exc>, retry: number, errorCount: number, error: any, request?: Req): void;
    /** Implemented by subclass to prepend/append data to request. */
    protected abstract masterSetupRequest(functionCode: pdu.EFunctionCode, request: Buffer): Req;
    /** Implemented by subclass to filter responses based on request. */
    protected abstract masterMatchResponse(request: Req, response: Res | Exc): boolean;
    /** Implemented by subclass to parse received data, returns length of parsed data. */
    protected abstract masterParseResponse(data: Buffer): number;
    /** Reset master internal state, must be called by subclass when (re)creating a connection. */
    protected masterReset(): void;
    /** Complete internal observable state, must be called by subclass on destruction. */
    protected masterDestroy(): void;
    /**
     * Receive data into internal buffer and try to parse responses.
     * Must be called by subclass after receiving data via connection for internal buffer management.
     * If a response is succesfully parsed then `masterOnResponse` shall be called with the result.
     */
    protected masterOnData(data: Buffer): void;
    /**
     * Response or exception parsed from incoming data by subclass.
     * Must be called by subclass after a response is successfully parsed from data.
     */
    protected masterOnResponse(response: Res | Exc): void;
    /** Validate number of retries (0+). */
    protected isRetry(value?: number): number;
    /** Validate timeout in milliseconds (50+). */
    protected isTimeout(value?: number): number;
    /** Get retryWhen callback. */
    protected isRetryWhen(callback?: IMasterRetryWhen<Req, Res, Exc>): IMasterRetryWhen<Req, Res, Exc>;
    /** Master queue handler for requests. */
    protected masterOnQueue(request: Req, options: IMasterRequestOptions<Req, Res, Exc, L>): Observable<Res>;
    /** Master handler for requests, sends request, waits for and matches responses. */
    protected masterOnRequest(id: number, request: Req, options: IMasterRequestOptions<Req, Res, Exc, L>): Observable<IMasterQueueOut<Res>>;
}
