/// <reference types="node" />
import * as pdu from "./pdu";
/**
 * Modbus PDU master.
 * Request factory for supported function codes.
 * TODO(L): Implement more Modbus function codes.
 */
export declare class Master {
    /**
     * Create a new PDU exception.
     * @param functionCode Function code.
     * @param exceptionCode Exception code.
     */
    static createException(functionCode: pdu.EFunctionCode, exceptionCode: pdu.EExceptionCode): pdu.Exception;
    /**
     * This function code is used to read from 1 to 2000 contiguous status of coils in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfCoils Quantity of coils.
     */
    static readCoils(startingAddress: number, quantityOfCoils: number): pdu.Request;
    /**
     * This function code is used to read from 1 to 2000 contiguous status of discrete inputs in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfInputs Quantity of inputs.
     */
    static readDiscreteInputs(startingAddress: number, quantityOfInputs: number): pdu.Request;
    /**
     * This function code is used to read  the contents of a contiguous block of holding registers in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfRegisters Quantity of registers.
     */
    static readHoldingRegisters(startingAddress: number, quantityOfRegisters: number): pdu.Request;
    /**
     * This function code is used to read from 1 to 125 contiguous input registers in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfRegisters Quantity of registers.
     */
    static readInputRegisters(startingAddress: number, quantityOfRegisters: number): pdu.Request;
    /**
     * This function code is used to write a single output to either ON or OFF in a remote device.
     * @param outputAddress Output address.
     * @param outputValue  Output value.
     */
    static writeSingleCoil(outputAddress: number, outputValue: boolean): pdu.Request;
    /**
     * This function code is used to write a single holding register in a remote device.
     * @param registerAddress Register address.
     * @param registerValue Register value.
     */
    static writeSingleRegister(registerAddress: number, registerValue: number): pdu.Request;
    /**
     * This function code is used to force each coil in a sequence of coils to either ON or OFF in a remote device.
     * @param startingAddress Starting address.
     * @param outputValues Output values.
     */
    static writeMultipleCoils(startingAddress: number, outputValues: boolean[]): pdu.Request;
    /**
     * This function code is used to write a block of contiguous registers (1 to 123 registers) in a remote device.
     * @param startingAddress Starting address.
     * @param registerValues Register values.
     */
    static writeMultipleRegisters(startingAddress: number, registerValues: number[]): pdu.Request;
    /**
     * Parse response buffer into PDU response or exception.
     * @param buffer Response buffer.
     */
    static onResponse(buffer: Buffer): pdu.Response | pdu.Exception;
    static onReadBitsResponse(buffer: Buffer): pdu.IReadBits;
    static onReadRegistersResponse(buffer: Buffer): pdu.IReadRegisters;
    static onWriteBitResponse(buffer: Buffer): pdu.IWriteBit;
    static onWriteRegisterResponse(buffer: Buffer): pdu.IWriteRegister;
    static onWriteMultipleResponse(buffer: Buffer): pdu.IWriteMultiple;
}
