"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:no-bitwise
var pdu = require("./pdu");
/**
 * Modbus PDU master.
 * Request factory for supported function codes.
 * TODO(L): Implement more Modbus function codes.
 */
var Master = /** @class */ (function () {
    function Master() {
    }
    /**
     * Create a new PDU exception.
     * @param functionCode Function code.
     * @param exceptionCode Exception code.
     */
    Master.createException = function (functionCode, exceptionCode) {
        var exceptionFunctionCode = (functionCode + 0x80) % 0xff;
        var buffer = Buffer.allocUnsafe(2);
        buffer.writeUInt8(exceptionFunctionCode, 0);
        buffer.writeUInt8(exceptionCode, 1);
        return new pdu.Exception(functionCode, functionCode + 0x80, exceptionCode, buffer);
    };
    /**
     * This function code is used to read from 1 to 2000 contiguous status of coils in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfCoils Quantity of coils.
     */
    Master.readCoils = function (startingAddress, quantityOfCoils) {
        pdu.isAddress(startingAddress);
        pdu.isQuantityOfBits(quantityOfCoils);
        pdu.isAddress(startingAddress + quantityOfCoils);
        var functionCode = pdu.EFunctionCode.ReadCoils;
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(startingAddress, 1);
        buffer.writeUInt16BE(quantityOfCoils, 3);
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * This function code is used to read from 1 to 2000 contiguous status of discrete inputs in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfInputs Quantity of inputs.
     */
    Master.readDiscreteInputs = function (startingAddress, quantityOfInputs) {
        pdu.isAddress(startingAddress);
        pdu.isQuantityOfBits(quantityOfInputs);
        pdu.isAddress(startingAddress + quantityOfInputs);
        var functionCode = pdu.EFunctionCode.ReadDiscreteInputs;
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(startingAddress, 1);
        buffer.writeUInt16BE(quantityOfInputs, 3);
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * This function code is used to read  the contents of a contiguous block of holding registers in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfRegisters Quantity of registers.
     */
    Master.readHoldingRegisters = function (startingAddress, quantityOfRegisters) {
        pdu.isAddress(startingAddress);
        pdu.isQuantityOfRegisters(quantityOfRegisters);
        pdu.isAddress(startingAddress + quantityOfRegisters);
        var functionCode = pdu.EFunctionCode.ReadHoldingRegisters;
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(startingAddress, 1);
        buffer.writeUInt16BE(quantityOfRegisters, 3);
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * This function code is used to read from 1 to 125 contiguous input registers in a remote device.
     * @param startingAddress Starting address.
     * @param quantityOfRegisters Quantity of registers.
     */
    Master.readInputRegisters = function (startingAddress, quantityOfRegisters) {
        pdu.isAddress(startingAddress);
        pdu.isQuantityOfRegisters(quantityOfRegisters);
        pdu.isAddress(startingAddress + quantityOfRegisters);
        var functionCode = pdu.EFunctionCode.ReadInputRegisters;
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(startingAddress, 1);
        buffer.writeUInt16BE(quantityOfRegisters, 3);
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * This function code is used to write a single output to either ON or OFF in a remote device.
     * @param outputAddress Output address.
     * @param outputValue  Output value.
     */
    Master.writeSingleCoil = function (outputAddress, outputValue) {
        pdu.isAddress(outputAddress);
        var functionCode = pdu.EFunctionCode.WriteSingleCoil;
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(outputAddress, 1);
        buffer.writeUInt16BE(!!outputValue ? 0xff00 : 0x0000, 3);
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * This function code is used to write a single holding register in a remote device.
     * @param registerAddress Register address.
     * @param registerValue Register value.
     */
    Master.writeSingleRegister = function (registerAddress, registerValue) {
        pdu.isAddress(registerAddress);
        pdu.isRegister(registerValue);
        var functionCode = pdu.EFunctionCode.WriteSingleRegister;
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(registerAddress, 1);
        buffer.writeUInt16BE(registerValue, 3);
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * This function code is used to force each coil in a sequence of coils to either ON or OFF in a remote device.
     * @param startingAddress Starting address.
     * @param outputValues Output values.
     */
    Master.writeMultipleCoils = function (startingAddress, outputValues) {
        pdu.isAddress(startingAddress);
        pdu.isQuantityOfBits(outputValues.length, 0x7b0);
        pdu.isAddress(startingAddress + outputValues.length);
        var functionCode = pdu.EFunctionCode.WriteMultipleCoils;
        var _a = pdu.bitsToBytes(outputValues), byteCount = _a[0], byteValues = _a[1];
        var buffer = Buffer.allocUnsafe(6 + byteCount);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(startingAddress, 1);
        buffer.writeUInt16BE(outputValues.length, 3);
        buffer.writeUInt8(byteCount, 5);
        byteValues.map(function (value, index) {
            buffer.writeUInt8(value, 6 + index);
        });
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * This function code is used to write a block of contiguous registers (1 to 123 registers) in a remote device.
     * @param startingAddress Starting address.
     * @param registerValues Register values.
     */
    Master.writeMultipleRegisters = function (startingAddress, registerValues) {
        pdu.isAddress(startingAddress);
        pdu.isQuantityOfRegisters(registerValues.length, 0x7b);
        pdu.isAddress(startingAddress + registerValues.length);
        registerValues.map(function (value) {
            pdu.isRegister(value);
        });
        var functionCode = pdu.EFunctionCode.WriteMultipleRegisters;
        var byteCount = registerValues.length * 2;
        var buffer = Buffer.allocUnsafe(6 + byteCount);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(startingAddress, 1);
        buffer.writeUInt16BE(registerValues.length, 3);
        buffer.writeUInt8(byteCount, 5);
        registerValues.map(function (value, index) {
            buffer.writeUInt16BE(value, 6 + index * 2);
        });
        return new pdu.Request(functionCode, buffer);
    };
    /**
     * Parse response buffer into PDU response or exception.
     * @param buffer Response buffer.
     */
    Master.onResponse = function (buffer) {
        var functionCode = buffer.readUInt8(0);
        var response = buffer.slice(1);
        if (functionCode >= 0x80) {
            // Buffer contains an exception response.
            var exceptionCode = buffer.readUInt8(1);
            return Master.createException(functionCode - 0x80, exceptionCode);
        }
        switch (functionCode) {
            case pdu.EFunctionCode.ReadCoils:
            case pdu.EFunctionCode.ReadDiscreteInputs: {
                var data = Master.onReadBitsResponse(response);
                return new pdu.Response(functionCode, data, buffer);
            }
            case pdu.EFunctionCode.ReadHoldingRegisters:
            case pdu.EFunctionCode.ReadInputRegisters: {
                var data = Master.onReadRegistersResponse(response);
                return new pdu.Response(functionCode, data, buffer);
            }
            case pdu.EFunctionCode.WriteSingleCoil: {
                var data = Master.onWriteBitResponse(response);
                return new pdu.Response(functionCode, data, buffer);
            }
            case pdu.EFunctionCode.WriteSingleRegister: {
                var data = Master.onWriteRegisterResponse(response);
                return new pdu.Response(functionCode, data, buffer);
            }
            case pdu.EFunctionCode.WriteMultipleCoils:
            case pdu.EFunctionCode.WriteMultipleRegisters: {
                var data = Master.onWriteMultipleResponse(response);
                return new pdu.Response(functionCode, data, buffer);
            }
            default: {
                // Unsupported function code, convert to exception.
                return Master.createException(functionCode, pdu.EExceptionCode.IllegalFunctionCode);
            }
        }
    };
    Master.onReadBitsResponse = function (buffer) {
        var bytes = buffer.readUInt8(0);
        var values = [];
        for (var i = 0; i < bytes; i++) {
            var byte = buffer.readUInt8(1 + i);
            for (var j = 0; j < 8; j++) {
                values.push(!!(byte & (0x1 << j)));
            }
        }
        return { bytes: bytes, values: values };
    };
    Master.onReadRegistersResponse = function (buffer) {
        var bytes = buffer.readUInt8(0);
        var values = [];
        for (var i = 0; i < bytes / 2; i++) {
            values[i] = buffer.readUInt16BE(1 + i * 2);
        }
        return { bytes: bytes, values: values };
    };
    Master.onWriteBitResponse = function (buffer) {
        var address = buffer.readUInt16BE(0);
        var value = buffer.readUInt16BE(2) === 0xff00 ? true : false;
        return { address: address, value: value };
    };
    Master.onWriteRegisterResponse = function (buffer) {
        var address = buffer.readUInt16BE(0);
        var value = buffer.readUInt16BE(2);
        return { address: address, value: value };
    };
    Master.onWriteMultipleResponse = function (buffer) {
        var address = buffer.readUInt16BE(0);
        var quantity = buffer.readUInt16BE(2);
        return { address: address, quantity: quantity };
    };
    return Master;
}());
exports.Master = Master;
//# sourceMappingURL=master.js.map