/// <reference types="node" />
/** Modbus function codes. */
export declare enum EFunctionCode {
    ReadCoils = 1,
    ReadDiscreteInputs = 2,
    ReadHoldingRegisters = 3,
    ReadInputRegisters = 4,
    WriteSingleCoil = 5,
    WriteSingleRegister = 6,
    WriteMultipleCoils = 15,
    WriteMultipleRegisters = 16,
    Mei = 43
}
/** Modbus exception codes. */
export declare enum EExceptionCode {
    IllegalFunctionCode = 1,
    IllegalDataAddress = 2,
    IllegalDataValue = 3,
    ServerFailure = 4,
    Acknowledge = 5,
    ServerBusy = 6
}
/** Modbus read coils/discrete inputs common interface. */
export interface IReadBits {
    readonly bytes: number;
    readonly values: boolean[];
}
export interface IReadCoils extends IReadBits {
}
export interface IReadDiscreteInputs extends IReadBits {
}
/** Modbus read holding/input registers common interface. */
export interface IReadRegisters {
    readonly bytes: number;
    readonly values: number[];
}
export interface IReadHoldingRegisters extends IReadRegisters {
}
export interface IReadInputRegisters extends IReadRegisters {
}
/** Modbus write coil common interface. */
export interface IWriteBit {
    readonly address: number;
    readonly value: boolean;
}
export interface IWriteSingleCoil extends IWriteBit {
}
/** Modbus write register common interface. */
export interface IWriteRegister {
    readonly address: number;
    readonly value: number;
}
export interface IWriteSingleRegister extends IWriteRegister {
}
/** Modbus write multiple coils/registers common interface. */
export interface IWriteMultiple {
    readonly address: number;
    readonly quantity: number;
}
export interface IWriteMultipleCoils extends IWriteMultiple {
}
export interface IWriteMultipleRegisters extends IWriteMultiple {
}
/** Modbus PDU request. */
export declare class Request {
    readonly functionCode: number;
    readonly buffer: Buffer;
    protected readonly isRequest = true;
    constructor(functionCode: number, buffer: Buffer);
}
/** Modbus PDU response. */
export declare class Response {
    readonly functionCode: number;
    readonly data: any;
    readonly buffer: Buffer;
    protected readonly isResponse = true;
    constructor(functionCode: number, data: any, buffer: Buffer);
}
/** Modbus PDU exception. */
export declare class Exception {
    readonly functionCode: number;
    readonly exceptionFunctionCode: number;
    readonly exceptionCode: number;
    readonly buffer: Buffer;
    protected readonly isException = true;
    constructor(functionCode: number, exceptionFunctionCode: number, exceptionCode: number, buffer: Buffer);
}
/** Returns true if instance of Request. */
export declare function isRequest(request: any): request is Request;
/** Returns true if instance of Response. */
export declare function isResponse(response: any): response is Response;
/** Returns true if instance of Exception. */
export declare function isException(exception: any): exception is Exception;
/** Throw an error if value is not a valid address. */
export declare function isAddress(value: number): void;
/** Throw an error if value is not a valid register. */
export declare function isRegister(value: number): void;
/** Throw an error if value is not a valid quantity of bits. */
export declare function isQuantityOfBits(value: number, maximum?: number): void;
/** Throw an error if value is not a valid quantity of bits. */
export declare function isQuantityOfRegisters(value: number, maximum?: number): void;
/** Convert an array of boolean bits to an array of byte values. */
export declare function bitsToBytes(values: boolean[]): [number, number[]];
/** Convert an array of byte values to an array of boolean bits. */
export declare function bytesToBits(quantity: number, buffer: Buffer): [number, boolean[]];
