"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var master_1 = require("./master");
var pdu = require("./pdu");
/** Modbus abstract PDU slave. */
var Slave = /** @class */ (function () {
    function Slave() {
    }
    /**
     * Parse request buffer into PDU response or exception.
     * @param buffer Request buffer.
     */
    Slave.prototype.onRequest = function (buffer) {
        var functionCode = buffer.readUInt8(0);
        var request = buffer.slice(1);
        switch (functionCode) {
            case pdu.EFunctionCode.ReadCoils: {
                return this.onReadBitsRequest(functionCode, request, true);
            }
            case pdu.EFunctionCode.ReadDiscreteInputs: {
                return this.onReadBitsRequest(functionCode, request, false);
            }
            case pdu.EFunctionCode.ReadHoldingRegisters: {
                return this.onReadRegistersRequest(functionCode, request, true);
            }
            case pdu.EFunctionCode.ReadInputRegisters: {
                return this.onReadRegistersRequest(functionCode, request, false);
            }
            case pdu.EFunctionCode.WriteSingleCoil: {
                return this.onWriteSingleBitRequest(functionCode, request);
            }
            case pdu.EFunctionCode.WriteSingleRegister: {
                return this.onWriteSingleRegisterRequest(functionCode, request);
            }
            case pdu.EFunctionCode.WriteMultipleCoils: {
                return this.onWriteMultipleBitsRequest(functionCode, request);
            }
            case pdu.EFunctionCode.WriteMultipleRegisters: {
                return this.onWriteMultipleRegistersRequest(functionCode, request);
            }
            default: {
                // Unsupported function code, convert to exception.
                return master_1.Master.createException(functionCode, pdu.EExceptionCode.IllegalFunctionCode);
            }
        }
    };
    // TODO(M): Buffer value/data validation.
    Slave.prototype.onReadBitsRequest = function (functionCode, request, isCoils) {
        var startingAddress = request.readUInt16BE(0);
        var quantityOfBits = request.readUInt16BE(2);
        var values = isCoils
            ? this.readCoils(startingAddress, quantityOfBits)
            : this.readDiscreteInputs(startingAddress, quantityOfBits);
        var _a = pdu.bitsToBytes(values), bytes = _a[0], byteValues = _a[1];
        var buffer = Buffer.allocUnsafe(2 + bytes);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt8(bytes, 1);
        byteValues.map(function (value, index) {
            buffer.writeUInt8(value, 2 + index);
        });
        var data = { bytes: bytes, values: values };
        return new pdu.Response(functionCode, data, buffer);
    };
    Slave.prototype.onReadRegistersRequest = function (functionCode, request, isHolding) {
        var startingAddress = request.readUInt16BE(0);
        var quantityOfRegisters = request.readUInt16BE(2);
        var values = isHolding
            ? this.readHoldingRegisters(startingAddress, quantityOfRegisters)
            : this.readInputRegisters(startingAddress, quantityOfRegisters);
        var bytes = values.length * 2;
        var buffer = Buffer.allocUnsafe(2 + bytes);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt8(bytes, 1);
        values.map(function (value, index) {
            buffer.writeUInt16BE(value, 2 + index * 2);
        });
        var data = { bytes: bytes, values: values };
        return new pdu.Response(functionCode, data, buffer);
    };
    Slave.prototype.onWriteSingleBitRequest = function (functionCode, request) {
        var address = request.readUInt16BE(0);
        var outputValue = request.readUInt16BE(2) === 0xff00;
        var value = this.writeSingleCoil(address, outputValue);
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(address, 1);
        buffer.writeUInt16BE(value ? 0xff00 : 0x0, 3);
        var data = { address: address, value: value };
        return new pdu.Response(functionCode, data, buffer);
    };
    Slave.prototype.onWriteSingleRegisterRequest = function (functionCode, request) {
        var address = request.readUInt16BE(0);
        var outputValue = request.readUInt16BE(2);
        var value = this.writeSingleRegister(address, outputValue);
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(address, 1);
        buffer.writeUInt16BE(value, 3);
        var data = { address: address, value: value };
        return new pdu.Response(functionCode, data, buffer);
    };
    Slave.prototype.onWriteMultipleBitsRequest = function (functionCode, request) {
        var address = request.readUInt16BE(0);
        var quantityOfBits = request.readUInt16BE(2);
        var bitValues = pdu.bytesToBits(quantityOfBits, request.slice(5));
        var quantity = this.writeMultipleCoils(address, bitValues[1]);
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(address, 1);
        buffer.writeUInt16BE(quantity, 3);
        var data = { address: address, quantity: quantity };
        return new pdu.Response(functionCode, data, buffer);
    };
    Slave.prototype.onWriteMultipleRegistersRequest = function (functionCode, request) {
        var address = request.readUInt16BE(0);
        var quantityOfRegisters = request.readUInt16BE(2);
        var registerValues = [];
        for (var i = 0; i < quantityOfRegisters; i++) {
            registerValues.push(request.readUInt16BE(5 + i * 2));
        }
        var quantity = this.writeMultipleRegisters(address, registerValues);
        var buffer = Buffer.allocUnsafe(5);
        buffer.writeUInt8(functionCode, 0);
        buffer.writeUInt16BE(address, 1);
        buffer.writeUInt16BE(quantity, 3);
        var data = { address: address, quantity: quantity };
        return new pdu.Response(functionCode, data, buffer);
    };
    return Slave;
}());
exports.Slave = Slave;
//# sourceMappingURL=slave.js.map