/// <reference types="node" />
import { Socket } from "net";
import { Observable } from "rxjs";
import * as adu from "../adu";
import * as pdu from "../pdu";
import * as tcp from "../tcp";
/** Modbus TCP client log names. */
export declare enum EMasterLog {
    Connecting = "ModbusTcpClientConnecting",
    Connected = "ModbusTcpClientConnected",
    Disconnected = "ModbusTcpClientDisconnected",
    Error = "ModbusTcpClientError"
}
export declare class ClientLog extends adu.MasterLog<tcp.Request, tcp.Response, tcp.Exception> {
    connecting(host: string, port: number, unitId: number): void;
    connected(host: string, port: number, unitId: number): void;
    disconnected(host: string, port: number, unitId: number): void;
    error(error: adu.MasterError): void;
    packetsTransmitted(value: number): void;
    packetsReceived(value: number): void;
}
export declare type IClientRequestOptions = adu.IMasterRequestOptions<tcp.Request, tcp.Response, tcp.Exception, ClientLog>;
/** Modbus TCP client options. */
export interface IClientOptions extends IClientRequestOptions {
    host?: string;
    port?: number;
    unitId?: number;
    inactivityTimeout?: number;
    log?: ClientLog;
}
/** Modbus TCP client. */
export declare class Client extends adu.Master<tcp.Request, tcp.Response, tcp.Exception, ClientLog> {
    /** Host the client will connect to. */
    readonly host: string;
    /** Port the client will connect to. */
    readonly port: number;
    /** Identifier of a remote slave. */
    readonly unitId: number;
    /** Inactivity client timeout. */
    readonly inactivityTimeout: number;
    /** Is client connected. */
    get connected(): boolean;
    protected socket: Socket | null;
    protected isConnected: boolean;
    protected transactionId: number;
    /** Node socket connection options. */
    protected get connectionOptions(): {
        host: string;
        port: number;
    };
    /** Get next transaction identifier. */
    protected get nextTransactionId(): number;
    /**
     * Create TCP client instance.
     * @param options Client and default request options.
     */
    constructor(options: IClientOptions, logConstructor?: typeof tcp.ClientLog);
    /**
     * Connect the client to configured host:port.
     * Observable will emit next after connected, throw an error if
     * connection fails and complete when 'disconnect' is called.
     * Completes if no data transmitted or received after timeout.
     * @param options Request options.
     */
    connect(options?: IClientRequestOptions): Observable<any>;
    /** If connected, disconnect the client from the configured host:port. */
    disconnect(): void;
    destroy(): void;
    protected destroySocket(): void;
    protected writeSocket(request: tcp.Request): void;
    /**
     * Parse buffer into response.
     * Inheritors may overwrite this function to implement their own response handler.
     */
    protected onParseResponse(transactionId: number, unitId: number, pduBuffer: Buffer, aduBuffer: Buffer): tcp.Response | tcp.Exception | null;
    /** Construct and prepend MBAP header to PDU request buffer. */
    protected masterSetupRequest(functionCode: pdu.EFunctionCode, request: Buffer): tcp.Request;
    /** Match transaction and unit identifiers of incoming responses. */
    protected masterMatchResponse(request: tcp.Request, response: tcp.Response | tcp.Exception): boolean;
    protected masterParseResponse(data: Buffer): number;
}
